@@ -6,20 +6,29 @@
 # Copyright (c) 2013 Eslam Mostafa <cseslam@gmail.com>
 # Copyright (c) 2013 Shivani Poddar <shivani.poddar92@gmail.com>
 # Copyright (c) 2013 Sai Suman Prayaga <suman.sai14@gmail.com>
-# Copyright (c) 2013 Seif Lotfy <seif@lotfy.com>.
+# Copyright (c) 2013 Seif Lotfy <seif@lotfy.com>
 #
-# Gnome Music is free software; you can Public License as published by the
-# Free Software Foundation; either version 2 of the License, or (at your
-# option) any later version.
+# GNOME Music is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
-# Gnome Music is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-# for more details.
+# GNOME Music is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along
-# with Gnome Music; if not, write to the Free Software Foundation,
-# Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+# with GNOME Music; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# The GNOME Music authors hereby grant permission for non-GPL compatible
+# GStreamer plugins to be used and distributed together with GStreamer
+# and GNOME Music.  This permission is above and beyond the permissions
+# granted by the GPL license by which GNOME Music is covered.  If you
+# modify this code, you may extend this exception to your version of the
+# code, but you are not obligated to do so.  If you do not wish to do so,
+# delete this exception statement from your version.
 
 
 from gi.repository import Gtk, Gio, GLib, Gdk
@@ -34,7 +43,7 @@ class Application(Gtk.Application):
                                  application_id='org.gnome.Music',
                                  flags=Gio.ApplicationFlags.FLAGS_NONE)
         GLib.set_application_name(_('Music'))
-
+        GLib.set_prgname('gnome-music')
         cssProviderFile = Gio.File.new_for_uri('resource:///org/gnome/Music/application.css')
         cssProvider = Gtk.CssProvider()
         cssProvider.load_from_file(cssProviderFile)
