@@ -3,18 +3,27 @@
 # Copyright (c) 2013 Seif Lotfy <seif@lotfy.com>
 # Copyright (c) 2013 Guillaume Quintard <guillaume.quintard@gmail.com>
 #
-# Gnome Music is free software; you can Public License as published by the
-# Free Software Foundation; either version 2 of the License, or (at your
-# option) any later version.
+# GNOME Music is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
-# Gnome Music is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-# for more details.
+# GNOME Music is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along
-# with Gnome Music; if not, write to the Free Software Foundation,
-# Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+# with GNOME Music; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# The GNOME Music authors hereby grant permission for non-GPL compatible
+# GStreamer plugins to be used and distributed together with GStreamer
+# and GNOME Music.  This permission is above and beyond the permissions
+# granted by the GPL license by which GNOME Music is covered.  If you
+# modify this code, you may extend this exception to your version of the
+# code, but you are not obligated to do so.  If you do not wish to do so,
+# delete this exception statement from your version.
 
 
 from gi.repository import Grl, GLib, GObject
@@ -81,8 +90,8 @@ class Grilo(GObject.GObject):
     def populate_albums(self, offset, callback, count=50):
         self.populate_items(Query.ALBUMS, offset, callback, count)
 
-    def populate_songs(self, offset, callback, count=50):
-        self.populate_items(Query.SONGS, offset, callback)
+    def populate_songs(self, offset, callback, count=-1):
+        self.populate_items(Query.SONGS, offset, callback, count)
 
     def populate_album_songs(self, album_id, callback):
         self.populate_items(Query.album_songs(album_id), 0, callback)
@@ -90,7 +99,8 @@ class Grilo(GObject.GObject):
     def populate_items(self, query, offset, callback, count=50):
         options = self.options.copy()
         options.set_skip(offset)
-        options.set_count(count)
+        if count != -1:
+            options.set_count(count)
 
         def _callback(source, param, item, count, data, offset):
             callback(source, param, item)
