@@ -1,18 +1,27 @@
 # Copyright (c) 2013 Arnel A. Borja <kyoushuu@yahoo.com>
 # Copyright (c) 2013 Vadim Rutkovsky <vrutkovs@redhat.com>
 #
-# Gnome Music is free software; you can Public License as published by the
-# Free Software Foundation; either version 2 of the License, or (at your
-# option) any later version.
+# GNOME Music is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
-# Gnome Music is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-# for more details.
+# GNOME Music is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along
-# with Gnome Music; if not, write to the Free Software Foundation,
-# Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+# with GNOME Music; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# The GNOME Music authors hereby grant permission for non-GPL compatible
+# GStreamer plugins to be used and distributed together with GStreamer
+# and GNOME Music.  This permission is above and beyond the permissions
+# granted by the GPL license by which GNOME Music is covered.  If you
+# modify this code, you may extend this exception to your version of the
+# code, but you are not obligated to do so.  If you do not wish to do so,
+# delete this exception statement from your version.
 
 
 import dbus
@@ -39,6 +48,7 @@ class MediaPlayer2Service(dbus.service.Object):
         self.player.connect('volume-changed', self._on_volume_changed)
         self.player.connect('prev-next-invalidated', self._on_prev_next_invalidated)
         self.player.connect('seeked', self._on_seeked)
+        self.first_song_handler = 0
 
     def _get_playback_status(self):
         state = self.player.get_playback_status()
@@ -138,6 +148,13 @@ class MediaPlayer2Service(dbus.service.Object):
                                },
                                [])
 
+    def _play_first_song(self, model, path, iter_, data=None):
+        if self.first_song_handler:
+            model.disconnect(self.first_song_handler)
+            self.first_song_handler = 0
+        self.player.set_playlist('Songs', None, model, iter_, 5)
+        self.player.set_playing(True)
+
     def _on_seeked(self, player, position, data=None):
         self.Seeked(position)
 
@@ -171,7 +188,17 @@ class MediaPlayer2Service(dbus.service.Object):
 
     @dbus.service.method(dbus_interface=MEDIA_PLAYER2_PLAYER_IFACE)
     def Play(self):
-        self.player.set_playing(True)
+        if self.player.playlist is not None:
+            self.player.set_playing(True)
+        elif self.first_song_handler == 0:
+            window = self.app.get_active_window()
+            window._stack.set_visible_child(window.views[2])
+            model = window.views[2]._model
+            if model.iter_n_children(None):
+                _iter = model.get_iter_first()
+                self._play_first_song(model, model.get_path(_iter), _iter)
+            else:
+                self.first_song_handler = model.connect('row-inserted', self._play_first_song)
 
     @dbus.service.method(dbus_interface=MEDIA_PLAYER2_PLAYER_IFACE,
                          in_signature='x')
