@@ -6,18 +6,27 @@
 # Copyright (c) 2013 Shivani Poddar <shivani.poddar92@gmail.com>
 # Copyright (c) 2013 Guillaume Quintard <guillaume.quintard@gmail.com>
 #
-# Gnome Music is free software; you can Public License as published by the
-# Free Software Foundation; either version 2 of the License, or (at your
-# option) any later version.
+# GNOME Music is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
-# Gnome Music is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-# for more details.
+# GNOME Music is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along
-# with Gnome Music; if not, write to the Free Software Foundation,
-# Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+# with GNOME Music; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# The GNOME Music authors hereby grant permission for non-GPL compatible
+# GStreamer plugins to be used and distributed together with GStreamer
+# and GNOME Music.  This permission is above and beyond the permissions
+# granted by the GPL license by which GNOME Music is covered.  If you
+# modify this code, you may extend this exception to your version of the
+# code, but you are not obligated to do so.  If you do not wish to do so,
+# delete this exception statement from your version.
 
 
 from gi.repository import GIRepository
@@ -72,7 +81,12 @@ class Player(GObject.GObject):
         self._symbolicIcon = self.cache.make_default_icon(ART_SIZE, ART_SIZE)
 
         Gst.init(None)
+
         self.discoverer = GstPbutils.Discoverer()
+        self.discoverer.connect('discovered', self._on_discovered)
+        self.discoverer.start()
+        self._discovering_urls = {}
+
         self.player = Gst.ElementFactory.make('playbin', 'player')
         self.bus = self.player.get_bus()
         self.bus.add_signal_watch()
@@ -86,6 +100,36 @@ class Player(GObject.GObject):
         self.bus.connect('message::eos', self._on_bus_eos)
         self._setup_view()
 
+        self.playlist_insert_handler = 0
+        self.playlist_delete_handler = 0
+
+    def discover_item(self, item, callback, data=None):
+        url = item.get_url()
+        if not url:
+            return
+
+        obj = (callback, data)
+
+        if url in self._discovering_urls:
+            self._discovering_urls[url] += [obj]
+        else:
+            self._discovering_urls[url] = [obj]
+            self.discoverer.discover_uri_async(url)
+
+    def _on_discovered(self, discoverer, info, error):
+        try:
+            cbs = self._discovering_urls[info.get_uri()]
+            del(self._discovering_urls[info.get_uri()])
+
+            for callback, data in cbs:
+                if data is not None:
+                    callback(info, error, data)
+                else:
+                    callback(info, error)
+        except KeyError:
+            # Not something we're interested in
+            return
+
     def _on_settings_changed(self, settings, value):
         self.repeat = settings.get_enum('repeat')
         self._sync_prev_next()
@@ -126,7 +170,11 @@ class Player(GObject.GObject):
             self.progressScale.set_value(0)
             self.progressScale.set_sensitive(False)
             if self.playlist is not None:
-                self.currentTrack = self.playlist.get_iter_first()
+                currentTrack = self.playlist.get_path(self.playlist.get_iter_first())
+                if currentTrack:
+                    self.currentTrack = Gtk.TreeRowReference.new(self.playlist, currentTrack)
+                else:
+                    self.currentTrack = None
                 self.load(self.get_current_media())
         else:
             #Stop playback
@@ -140,8 +188,11 @@ class Player(GObject.GObject):
         self.currentTrack = self.nextTrack
         self.play()
 
+    def _on_playlist_size_changed(self, path, _iter=None, data=None):
+        self._sync_prev_next()
+
     def _get_next_track(self):
-        currentTrack = self.currentTrack
+        currentTrack = self.playlist.get_iter(self.currentTrack.get_path())
         nextTrack = None
         if self.repeat == RepeatType.SONG:
             nextTrack = currentTrack
@@ -157,7 +208,10 @@ class Player(GObject.GObject):
             for i in range(1, randint(1, rows)):
                 nextTrack = self.playlist.iter_next(nextTrack)
 
-        return nextTrack
+        if nextTrack:
+            return Gtk.TreeRowReference.new(self.playlist, self.playlist.get_path(nextTrack))
+        else:
+            return None
 
     def _get_iter_last(self):
         iter = self.playlist.get_iter_first()
@@ -170,7 +224,7 @@ class Player(GObject.GObject):
         return last
 
     def _get_previous_track(self):
-        currentTrack = self.currentTrack
+        currentTrack = self.playlist.get_iter(self.currentTrack.get_path())
         previousTrack = None
 
         if self.repeat == RepeatType.SONG:
@@ -187,7 +241,10 @@ class Player(GObject.GObject):
             for i in range(1, randint(1, rows)):
                 previousTrack = self.playlist.iter_next(previousTrack)
 
-        return previousTrack
+        if previousTrack:
+            return Gtk.TreeRowReference.new(self.playlist, self.playlist.get_path(previousTrack))
+        else:
+            return None
 
     def has_next(self):
         if not self.currentTrack:
@@ -195,7 +252,7 @@ class Player(GObject.GObject):
         elif self.repeat in [RepeatType.ALL, RepeatType.SONG, RepeatType.SHUFFLE]:
             return True
         else:
-            tmp = self.currentTrack.copy()
+            tmp = self.playlist.get_iter(self.currentTrack.get_path())
             return self.playlist.iter_next(tmp) is not None
 
     def has_previous(self):
@@ -204,7 +261,7 @@ class Player(GObject.GObject):
         elif self.repeat in [RepeatType.ALL, RepeatType.SONG, RepeatType.SHUFFLE]:
             return True
         else:
-            tmp = self.currentTrack.copy()
+            tmp = self.playlist.get_iter(self.currentTrack.get_path())
             return self.playlist.iter_previous(tmp) is not None
 
     def _get_playing(self):
@@ -265,7 +322,8 @@ class Player(GObject.GObject):
         if url != self.player.get_value('current-uri', 0):
             self.player.set_property('uri', url)
 
-        self.emit('playlist-item-changed', self.playlist, self.currentTrack)
+        currentTrack = self.playlist.get_iter(self.currentTrack.get_path())
+        self.emit('playlist-item-changed', self.playlist, currentTrack)
         self.emit('current-changed')
 
     def _on_cache_lookup(self, pixbuf, path, data=None):
@@ -340,11 +398,20 @@ class Player(GObject.GObject):
     def set_playlist(self, type, id, model, iter, field):
         self.stop()
 
+        if self.playlist_insert_handler:
+            self.playlist.disconnect(self.playlist_insert_handler)
+        if self.playlist_delete_handler:
+            self.playlist.disconnect(self.playlist_delete_handler)
+
         self.playlist = model
         self.playlistType = type
         self.playlistId = id
-        self.currentTrack = iter
+        self.currentTrack = Gtk.TreeRowReference.new(model, model.get_path(iter))
         self.playlistField = field
+
+        self.playlist_insert_handler = model.connect('row-inserted', self._on_playlist_size_changed)
+        self.playlist_delete_handler = model.connect('row-deleted', self._on_playlist_size_changed)
+
         self.emit('current-changed')
 
     def running_playlist(self, type, id):
@@ -530,7 +597,8 @@ class Player(GObject.GObject):
     def get_current_media(self):
         if not self.currentTrack:
             return None
-        return self.playlist.get_value(self.currentTrack, self.playlistField)
+        currentTrack = self.playlist.get_iter(self.currentTrack.get_path())
+        return self.playlist.get_value(currentTrack, self.playlistField)
 
 
 class SelectionToolbar():
