@@ -8,18 +8,27 @@
 # Copyright (c) 2013 Jackson Isaac <jacksonisaac2008@gmail.com>
 # Copyright (c) 2013 Guillaume Quintard <guillaume.quintard@gmail.com>
 #
-# Gnome Music is free software; you can Public License as published by the
-# Free Software Foundation; either version 2 of the License, or (at your
-# option) any later version.
+# GNOME Music is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
-# Gnome Music is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-# for more details.
+# GNOME Music is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along
-# with Gnome Music; if not, write to the Free Software Foundation,
-# Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+# with GNOME Music; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# The GNOME Music authors hereby grant permission for non-GPL compatible
+# GStreamer plugins to be used and distributed together with GStreamer
+# and GNOME Music.  This permission is above and beyond the permissions
+# granted by the GPL license by which GNOME Music is covered.  If you
+# modify this code, you may extend this exception to your version of the
+# code, but you are not obligated to do so.  If you do not wish to do so,
+# delete this exception statement from your version.
 
 
 from gi.repository import Gtk
@@ -30,6 +39,7 @@ from gi.repository import Pango
 from gi.repository import GLib
 from gi.repository import GdkPixbuf
 from gi.repository import Tracker
+from gi.repository import Gio
 from gettext import gettext as _
 from gnomemusic.grilo import grilo
 import gnomemusic.widgets as Widgets
@@ -126,6 +136,8 @@ class ViewContainer(Stack):
         self.view.connect('view-selection-changed',
                           self._on_view_selection_changed)
 
+        self._discovering_urls = {}
+
     def _get_remaining_item_count(self):
         if self._cached_count < 0:
             self._cached_count = Widgets.get_count(self.countQuery)
@@ -200,6 +212,10 @@ class ViewContainer(Stack):
     def populate(self):
         print('populate')
 
+    def _on_discovered(self, info, error, _iter):
+        if error:
+            self._model.set(_iter, [8, 10], [self.errorIconName, True])
+
     def _add_item(self, source, param, item):
         if not item:
             return
@@ -210,23 +226,15 @@ class ViewContainer(Stack):
         title = albumArtCache.get_media_title(item)
         item.set_title(title)
 
-        def add_new_item():
-            _iter = self._model.append()
-            icon_name = self.nowPlayingIconName
-            if item.get_url():
-                try:
-                    self.player.discoverer.discover_uri(item.get_url())
-                except:
-                    print('failed to discover url ' + item.get_url())
-                    icon_name = self.errorIconName
-            self._model.set(_iter,
-                            [0, 1, 2, 3, 4, 5, 7, 8, 9, 10],
-                            [str(item.get_id()), '', title,
-                             artist, self._symbolicIcon, item,
-                             -1, icon_name, False, icon_name == self.errorIconName])
-            GLib.idle_add(self._update_album_art, item, _iter)
-
-        GLib.idle_add(add_new_item)
+        icon_name = self.nowPlayingIconName
+        _iter = self._model.insert_with_valuesv(
+            -1,
+            [0, 1, 2, 3, 4, 5, 7, 8, 9, 10],
+            [str(item.get_id()), '', title,
+             artist, self._symbolicIcon, item,
+             -1, icon_name, False, False])
+        self.player.discover_item(item, self._on_discovered, _iter)
+        GLib.idle_add(self._update_album_art, item, _iter)
 
     def _insert_album_art(self, item, cb_item, itr, x=False):
         if item and cb_item and not item.get_thumbnail():
@@ -265,7 +273,10 @@ class Empty(Stack):
                        transition_type=StackTransitionType.CROSSFADE)
         builder = Gtk.Builder()
         builder.add_from_resource('/org/gnome/Music/NoMusic.ui')
+        music_folder_path = GLib.get_user_special_dir(GLib.USER_DIRECTORY_MUSIC)
         widget = builder.get_object('container')
+        label = builder.get_object('label1')
+        label.set_label(_("No Music found!\n Put some files into the folder %s" %music_folder_path))
         self.add(widget)
         self.show_all()
 
@@ -277,6 +288,7 @@ class Albums(ViewContainer):
         self.view.set_view_type(Gd.MainViewType.ICON)
         self.countQuery = Query.ALBUMS_COUNT
         self._albumWidget = Widgets.AlbumWidget(player)
+        self.player = player
         self.add(self._albumWidget)
 
     def _back_button_clicked(self, widget, data=None):
@@ -305,6 +317,7 @@ class Songs(ViewContainer):
         ViewContainer.__init__(self, _("Songs"), header_bar, selection_toolbar)
         self.countQuery = Query.SONGS_COUNT
         self._items = {}
+        self.monitors = []
         self.isStarred = None
         self.iter_to_clean = None
         self.view.set_view_type(Gd.MainViewType.LIST)
@@ -340,23 +353,26 @@ class Songs(ViewContainer):
         if not item:
             return
         self._offset += 1
-        _iter = self._model.append()
         item.set_title(albumArtCache.get_media_title(item))
-        try:
-            if item.get_url():
-                self.player.discoverer.discover_uri(item.get_url())
-            self._model.set(_iter,
-                            [2, 3, 5, 8, 9, 10],
-                            [albumArtCache.get_media_title(item),
-                             item.get_string(Grl.METADATA_KEY_ARTIST),
-                             item, self.nowPlayingIconName, False, False])
-        except:
-            print('failed to discover url ' + item.get_url())
-            self._model.set(_iter,
-                            [2, 3, 5, 8, 9, 10],
-                            [albumArtCache.get_media_title(item),
-                             item.get_string(Grl.METADATA_KEY_ARTIST),
-                             item, self.errorIconName, False, True])
+        artist = item.get_string(Grl.METADATA_KEY_ARTIST)\
+            or item.get_author()\
+            or _("Unknown Artist")
+        _iter = self._model.insert_with_valuesv(
+            -1,
+            [2, 3, 5, 8, 9, 10],
+            [albumArtCache.get_media_title(item),
+             artist, item, self.nowPlayingIconName, False, False])
+        self.player.discover_item(item, self._on_discovered, _iter)
+        g_file = Gio.file_new_for_uri(item.get_url())
+        self.monitors.append(g_file.monitor_file(Gio.FileMonitorFlags.NONE,
+                                                 None))
+        self.monitors[(self._offset - 1)].connect('changed',
+                                                  self._on_item_changed, _iter)
+
+    def _on_item_changed(self, monitor, file1, file2, event, _iter):
+        if self._model.iter_is_valid(_iter):
+            if event == Gio.FileMonitorEvent.DELETED:
+                self._model.set(_iter, [8, 10], [self.errorIconName, True])
 
     def _add_list_renderers(self):
         list_widget = self.view.get_generic_view()
@@ -489,11 +505,10 @@ class Artists (ViewContainer):
     def _populate(self, data=None):
         selection = self.view.get_generic_view().get_selection()
         if not selection.get_selected()[1]:
-            self._allIter = self._model.append()
+            self._allIter = self._model.insert_with_valuesv(-1, [2], [_("All Artists")])
             self._last_selection = self._allIter
             self._artists[_("All Artists").lower()] =\
                 {'iter': self._allIter, 'albums': []}
-            self._model.set(self._allIter, 2, _("All Artists"))
             selection.select_path(self._model.get_path(self._allIter))
             self.view.emit('item-activated', '0',
                            self._model.get_path(self._allIter))
@@ -552,15 +567,12 @@ class Artists (ViewContainer):
         if item is None:
             return
         self._offset += 1
-        artist = item.get_string(Grl.METADATA_KEY_ARTIST)
-        if not artist:
-            artist = item.get_author()
-        if not artist:
-            artist = _("Unknown Artist")
+        artist = item.get_string(Grl.METADATA_KEY_ARTIST)\
+            or item.get_author()\
+            or _("Unknown Artist")
         if not artist.lower() in self._artists:
-            _iter = self._model.append()
+            _iter = self._model.insert_with_valuesv(-1, [2], [artist])
             self._artists[artist.lower()] = {'iter': _iter, 'albums': []}
-            self._model.set(_iter, [2], [artist])
 
         self._artists[artist.lower()]['albums'].append(item)
 
