@@ -8,27 +8,35 @@
 # Copyright (c) 2013 Giovanni Campagna <scampa.giovanni@gmail.com>
 # Copyright (c) 2013 Guillaume Quintard <guillaume.quintard@gmail.com>
 #
-# Gnome Music is free software; you can Public License as published by the
-# Free Software Foundation; either version 2 of the License, or (at your
-# option) any later version.
+# GNOME Music is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
-# Gnome Music is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-# for more details.
+# GNOME Music is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along
-# with Gnome Music; if not, write to the Free Software Foundation,
-# Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+# with GNOME Music; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# The GNOME Music authors hereby grant permission for non-GPL compatible
+# GStreamer plugins to be used and distributed together with GStreamer
+# and GNOME Music.  This permission is above and beyond the permissions
+# granted by the GPL license by which GNOME Music is covered.  If you
+# modify this code, you may extend this exception to your version of the
+# code, but you are not obligated to do so.  If you do not wish to do so,
+# delete this exception statement from your version.
 
 
 from gi.repository import Gtk, Gd, GLib, GObject, Pango
-from gi.repository import GdkPixbuf
+from gi.repository import GdkPixbuf, Gio
 from gi.repository import Grl
 from gi.repository import Tracker
 from gettext import gettext as _
 from gnomemusic.grilo import grilo
-import logging
 from gnomemusic.query import Query
 from gnomemusic.albumArtCache import AlbumArtCache
 
@@ -115,7 +123,7 @@ class AlbumWidget(Gtk.EventBox):
         self.view.set_view_type(Gd.MainViewType.LIST)
         self.album = None
         self.view.connect('item-activated', self._on_item_activated)
-
+        self.monitors = []
         view_box = self.ui.get_object('view')
         self.ui.get_object('scrolledWindow').set_placement(Gtk.CornerType.
                                                            TOP_LEFT)
@@ -213,9 +221,11 @@ class AlbumWidget(Gtk.EventBox):
         cached_playlist = self.player.running_playlist('Album', album)
         if cached_playlist:
             self.model = cached_playlist
+            currentTrack = self.player.playlist.get_iter(self.player.currentTrack.get_path())
             self.update_model(self.player, cached_playlist,
-                              self.player.currentTrack)
+                              currentTrack)
         else:
+            self.duration = 0
             self._create_model()
             GLib.idle_add(grilo.populate_album_songs, item.get_id(),
                           self._on_populate_album_songs)
@@ -263,31 +273,36 @@ class AlbumWidget(Gtk.EventBox):
             if(self.player.get_playback_status() != 2):
                 self.player.eventBox.set_visible(True)
 
+    def _on_discovered(self, info, error, _iter):
+        if error:
+            self.model.set(_iter, [7, 9], [ERROR_ICON_NAME, True])
+
     def _on_populate_album_songs(self, source, prefs, track):
         if track:
             self.tracks.append(track)
             self.duration = self.duration + track.get_duration()
             _iter = self.model.append()
+            self.player.discover_item(track, self._on_discovered, _iter)
+            g_file = Gio.file_new_for_uri(track.get_url())
+            self.monitors.append(g_file.monitor_file(Gio.FileMonitorFlags.NONE,
+                                                     None))
+            self.monitors[-1].connect('changed', self._on_item_changed, _iter)
             escapedTitle = AlbumArtCache.get_media_title(track, True)
-            try:
-                self.player.discoverer.discover_uri(track.get_url())
-                self.model.set(_iter,
-                               [0, 1, 2, 3, 4, 5, 7, 9],
-                               [escapedTitle,
-                                self.player.seconds_to_string(
-                                    track.get_duration()),
-                                '', '', None, track, NOW_PLAYING_ICON_NAME,
-                                False])
-            except Exception as err:
-                logging.debug(err.message)
-                logging.debug('failed to discover url ' + track.get_url())
-                self.model.set(_iter,
-                               [0, 1, 2, 3, 4, 5, 7, 9],
-                               [escapedTitle, '', '', '', None,
-                                track, ERROR_ICON_NAME, True])
+            self.model.set(_iter,
+                           [0, 1, 2, 3, 4, 5, 7, 9],
+                           [escapedTitle,
+                            self.player.seconds_to_string(
+                                track.get_duration()),
+                            '', '', None, track, NOW_PLAYING_ICON_NAME,
+                            False])
             self.ui.get_object('running_length_label_info').set_text(
                 '%d min' % (int(self.duration / 60) + 1))
 
+    def _on_item_changed(self, monitor, file1, file2, event_type, _iter):
+        if self.model.iter_is_valid(_iter):
+            if event_type == Gio.FileMonitorEvent.DELETED:
+                self.model.set(_iter, [7, 9], [ERROR_ICON_NAME, True])
+
     def _on_look_up(self, pixbuf, path, data=None):
         _iter = self.iterToClean
         if pixbuf:
@@ -302,9 +317,10 @@ class AlbumWidget(Gtk.EventBox):
         currentSong = playlist.get_value(currentIter, 5)
         song_passed = False
         _iter = playlist.get_iter_first()
+        self.duration = 0
         while _iter:
             song = playlist.get_value(_iter, 5)
-
+            self.duration += song.get_duration()
             escapedTitle = AlbumArtCache.get_media_title(song, True)
             if (song == currentSong):
                 title = '<b>%s</b>' % escapedTitle
@@ -320,6 +336,8 @@ class AlbumWidget(Gtk.EventBox):
             if(playlist.get_value(_iter, 7) != ERROR_ICON_NAME):
                 playlist.set_value(_iter, 9, iconVisible)
             _iter = playlist.iter_next(_iter)
+            self.ui.get_object('running_length_label_info').set_text(
+                '%d min' % (int(self.duration / 60) + 1))
         return False
 
 
@@ -487,7 +505,7 @@ class ArtistAlbumWidget(Gtk.HBox):
         self.artist = album.get_string(Grl.METADATA_KEY_ARTIST)
         self.model = model
         self.songs = []
-
+        self.monitors = []
         self.ui = Gtk.Builder()
         self.ui.add_from_resource('/org/gnome/Music/ArtistAlbumWidget.ui')
 
@@ -508,6 +526,15 @@ class ArtistAlbumWidget(Gtk.HBox):
         self.pack_start(self.ui.get_object('ArtistAlbumWidget'), True, True, 0)
         self.show_all()
 
+    def _on_discovered(self, info, error, song_widget):
+        if error:
+            self.model.set(song_widget._iter, [4], [ERROR_ICON_NAME])
+            song_widget.now_playing_sign.set_from_icon_name(
+                ERROR_ICON_NAME,
+                Gtk.IconSize.SMALL_TOOLBAR)
+            song_widget.now_playing_sign.show()
+            song_widget.can_be_played = False
+
     def get_songs(self, source, prefs, track):
         if track:
             self.tracks.append(track)
@@ -533,36 +560,31 @@ class ArtistAlbumWidget(Gtk.HBox):
                 song_widget._iter = itr
                 song_widget.model = self.model
                 song_widget.title = ui.get_object('title')
-
-                try:
-                    self.player.discoverer.discover_uri(str(track.get_url()))
-                    self.model.set(itr,
-                                   [0, 1, 2, 3, 4, 5],
-                                   [title, '', '', False,
-                                    NOW_PLAYING_ICON_NAME, track])
-                    song_widget.now_playing_sign = ui.get_object('image1')
-                    song_widget.now_playing_sign.set_from_icon_name(
-                        NOW_PLAYING_ICON_NAME,
-                        Gtk.IconSize.SMALL_TOOLBAR)
-                    song_widget.now_playing_sign.set_no_show_all('True')
-                    song_widget.now_playing_sign.set_alignment(1, 0.6)
-                    song_widget.can_be_played = True
-                    song_widget.connect('button-release-event',
-                                        self.track_selected)
-
-                except:
-                    print('failed to discover url ' + str(track.get_url()))
-                    self.model.set(itr, [0, 1, 2, 3, 4, 5],
-                                   [title, '', '', True,
-                                    ERROR_ICON_NAME, track])
-                    song_widget.now_playing_sign = ui.get_object('image1')
-                    song_widget.now_playing_sign.set_from_icon_name(
-                        ERROR_ICON_NAME,
-                        Gtk.IconSize.SMALL_TOOLBAR)
-                    song_widget.now_playing_sign.set_alignment(1, 0.6)
-                    song_widget.can_be_played = False
+                self.player.discover_item(track, self._on_discovered, song_widget)
+                g_file = Gio.file_new_for_uri(track.get_url())
+                self.monitors.append(g_file.monitor_file(Gio.FileMonitorFlags.NONE,
+                                                         None))
+                self.monitors[-1].connect('changed', self._on_item_changed, itr)
+                self.model.set(itr,
+                               [0, 1, 2, 3, 4, 5],
+                               [title, '', '', False,
+                                NOW_PLAYING_ICON_NAME, track])
+                song_widget.now_playing_sign = ui.get_object('image1')
+                song_widget.now_playing_sign.set_from_icon_name(
+                    NOW_PLAYING_ICON_NAME,
+                    Gtk.IconSize.SMALL_TOOLBAR)
+                song_widget.now_playing_sign.set_no_show_all('True')
+                song_widget.now_playing_sign.set_alignment(1, 0.6)
+                song_widget.can_be_played = True
+                song_widget.connect('button-release-event',
+                                    self.track_selected)
             self.ui.get_object('grid1').show_all()
 
+    def _on_item_changed(self, monitor, file1, file2, event_type, _iter):
+        if self.model.iter_is_valid(_iter):
+            if event_type == Gio.FileMonitorEvent.DELETED:
+                self.model.set(_iter, [3, 4], [True, ERROR_ICON_NAME])
+
     def _update_album_art(self):
         ALBUM_ART_CACHE.lookup(self.album, 128, 128, self._get_album_cover)
 
@@ -571,6 +593,9 @@ class ArtistAlbumWidget(Gtk.HBox):
             self.ui.get_object('cover').set_from_pixbuf(pixbuf)
 
     def track_selected(self, widget, _iter):
+        if not widget.can_be_played:
+            return
+
         self.player.stop()
         self.player.set_playlist('Artist', self.album,
                                  widget.model, widget._iter, 5)
